package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.annotation.Lock;
import com.xy.config.ThreadPoolConfig;
import com.xy.dto.DeviceDataDto;
import com.xy.entity.DeviceData;
import com.xy.entity.SysDictRedis;
import com.xy.mapper.DeviceDataMapper;
import com.xy.utils.*;
import com.xy.utils.enums.DeviceEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;


/**
 * <p>
 * 设备统计数据 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-01-11
 */
@Service
@AllArgsConstructor
@Api(tags = "设备统计数据")
public class DeviceDataServiceImpl extends ServiceImpl<DeviceDataMapper, DeviceData> implements DeviceDataService {

    @PostMapping("obj")
    @ApiOperation("对象查询")
    public R<DeviceDataDto.Vo> obj(@RequestBody DeviceDataDto.Vo vo) {
        LambdaQueryWrapper<DeviceData> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(vo, DeviceData.class).build();
        List<DeviceData> list = list(lambdaQueryWrapper);
        if (Emptys.check(list)) {
            return R.ok(copy(DeviceDataDto.Vo.class, list).get(0));
        }
        return R.ok();
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    private R<PageBean<DeviceDataDto.Vo>> page(@RequestBody DeviceDataDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<DeviceData> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, DeviceData.class)
                .ge(DeviceData::getCreateTime, page.getBeginCreateTime())
                .le(DeviceData::getCreateTime, page.getEndCreateTime())
                .build()
                .orderByDesc(!Emptys.check(pageBean.getOrders()), DeviceData::getCreateTime);
        IPage<DeviceData> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        return R.ok(toPageBean(DeviceDataDto.Vo.class, iPage));
    }

    @PostMapping("saveOrAccum")
    @ApiOperation("添加/累加")
    @Transactional(rollbackFor = Exception.class)
    @Lock(value = "saveOrAccum.deviceId", prefix = "data_save_accum_")
    public R saveOrAccum(@RequestBody @Validated DeviceDataDto.SaveOrAccum saveOrAccum) {
        LocalDateTime now = LocalDateTime.now();
        String stringTime = DataTime.toString(now, "yyyyMMdd");
        Map<String, SysDictRedis> map = SysDictUtils.get(DeviceEnum.DEVICE_DATA_TYPE.getKey());
        ThreadPoolUtils.Execute execute = ThreadPoolUtils.excPoll(ThreadPoolConfig.DEVICE_DATA_SAVE_OR_ACCUM, map.size());
        map.forEach((type, sysDictRedis) -> execute.execute(() -> {
            Integer dateValue = Integer.valueOf(stringTime.substring(0, Integer.valueOf(sysDictRedis.getValue())));
            DeviceData saveOrUpdateInfo = copy(DeviceData.class, saveOrAccum);
            DeviceData deviceData = getOne(new LambdaQueryWrapper<DeviceData>()
                    .eq(DeviceData::getDeviceId, saveOrAccum.getDeviceId())
                    .eq(DeviceData::getType, type)
            );
            //添加
            if (deviceData == null) {
                saveOrUpdateInfo
                        .setCreateTime(now)
                        .setUpdateTime(now)
                        .setType(type)
                        .setDateValue(dateValue);
                save(saveOrUpdateInfo);
                return;
            }
            //累加
            DeviceData updateDeviceData = new DeviceData().setId(deviceData.getId()).setUpdateTime(now)
                    .setSalesMoney(saveOrAccum.getSalesMoney() != null ? saveOrAccum.getSalesMoney() + deviceData.getSalesMoney() : null)
                    .setSalesCount(saveOrAccum.getSalesCount() != null ? saveOrAccum.getSalesCount() + deviceData.getSalesCount() : null)
                    .setGoodsCount(saveOrAccum.getGoodsCount() != null ? saveOrAccum.getGoodsCount() + deviceData.getGoodsCount() : null)
                    .setRefundMoney(saveOrAccum.getRefundMoney() != null ? saveOrAccum.getRefundMoney() + deviceData.getRefundMoney() : null)
                    .setRefundCount(saveOrAccum.getRefundCount() != null ? saveOrAccum.getRefundCount() + deviceData.getRefundCount() : null)
                    .setZeroCount(saveOrAccum.getZeroCount() != null ? saveOrAccum.getZeroCount() + deviceData.getZeroCount() : null)
                    .setDeviceFaultCount(saveOrAccum.getDeviceFaultCount() != null ? saveOrAccum.getDeviceFaultCount() + deviceData.getDeviceFaultCount() : null);
            updateById(updateDeviceData);
        }));
        execute.end();
        return R.ok();
    }

}